@@ -549,12 +549,9 @@
                                               :initial-contents '(1 0 0))))))))
       ;; It would be bad if another thread sees MAKE-ARRAY's result in the
       ;; global variable before the vector's header+length have been set.
-      ;; This is theoretically possible if the architecture allows out-of-order
-      ;; memory writes.  A barrier will prevent that, but a meta-bug prevents
-      ;; using SB!THREAD:BARRIER here. The macro isn't defined yet? (FIXME)
-      ;; Note that this bug already existed and I'm just documenting it.
-      ;; Most likely all caches are made before ever starting multiple threads.
-      (progn ; sb!thread:barrier (:write)
+      ;; Without a barrier, this would be theoretically possible if the
+      ;; architecture allows out-of-order memory writes.
+      (sb!thread:barrier (:write)
         (reset-stats)
         (setq cache (make-array size :initial-element 0)))
       (set! symbol cache))))
@@ -591,8 +588,7 @@
   (def 5)
   (def 6))
 
-;; Should this be !DEFINE-HASH-CACHE ?
-(defmacro define-hash-cache (name args
+(defmacro !define-hash-cache (name args
                              &key hash-function hash-bits memoizer
                                   (values 1))
   (declare (ignore memoizer))
@@ -649,6 +645,10 @@
                      (let ((,entry (svref (truly-the ,cache-type ,cache)
                                           (ldb (byte ,hash-bits 0) ,hashval))))
                        (unless (eql ,entry 0)
+                         ;; This barrier is a no-op on all multi-threaded SBCL
+                         ;; architectures. No CPU except Alpha will move a read
+                         ;; prior to a read on which it depends.
+                         (sb!thread:barrier (:data-dependency))
                          (locally (declare (type ,line-type ,entry))
                            (let* ,binds
                              (when (and ,@tests)
@@ -668,6 +668,11 @@
                        (idx2 (ldb (byte ,hash-bits ,hash-bits) ,hashval)))
                    ,@(when *profile-hash-cache*
                        `((incf (aref ,statistics-name 1)))) ; count misses
+                   ;; Why a barrier: the pointer to 'entry' (a cons or vector)
+                   ;; MUST NOT be observed by another thread before its cells
+                   ;; are filled. Equally bad, the 'output' cells in the line
+                   ;; could be 0 while the 'input' cells matched something.
+                   (sb!thread:barrier (:write))
                    (cond ((eql (svref ,cache idx1) 0)
                           (setf (svref ,cache idx1) ,entry))
                          ((eql (svref ,cache idx2) 0)
@@ -689,8 +694,8 @@
        ,fun)))
 
 ;;; some syntactic sugar for defining a function whose values are
-;;; cached by DEFINE-HASH-CACHE
-;;; These keywords are mostly defined at DEFINE-HASH-CACHE.
+;;; cached by !DEFINE-HASH-CACHE
+;;; These keywords are mostly defined at !DEFINE-HASH-CACHE.
 ;;; Additional options:
 ;;; :MEMOIZER <name>
 ;;;   If provided, it is the name of a local macro that must be called
@@ -710,7 +715,7 @@
     ;; What I wouldn't give to be able to use BINDING*, right?
     (multiple-value-bind (forms decls doc) (parse-body body-decls-doc)
       `(progn
-        (define-hash-cache ,name ,args ,@options)
+        (!define-hash-cache ,name ,args ,@options)
         (defun ,name ,arg-names
           ,@decls
           ,@(if doc (list doc))
@@ -1140,17 +1145,54 @@
         :since since
         :runtime-error runtime-error))
 
-(defun deprecated-function (since name replacements)
-  (lambda (&rest deprecated-function-args)
-    (declare (ignore deprecated-function-args))
-    (deprecation-error since name replacements)))
+(defun deprecated-function (since name replacements &optional doc)
+  (let ((closure
+         ;; setting the name is mildly redundant since the closure captures
+         ;; its name. However %FUN-DOC can't make use of that fact.
+         (set-closure-name
+          (lambda (&rest deprecated-function-args)
+            (declare (ignore deprecated-function-args))
+            (deprecation-error since name replacements))
+          name)))
+    (when doc
+      (setf (%fun-doc closure) doc))
+    closure))
 
 (defun deprecation-compiler-macro (state since name replacements)
-  (lambda (form env)
+  ;; this lambda's name is significant - see DEPRECATED-THING-P
+  (named-lambda .deprecation-warning. (form env)
     (declare (ignore env))
     (deprecation-warning state since name replacements)
     form))
 
+;; Return the stage of deprecation of thing identified by KIND and NAME, or NIL.
+(defun deprecated-thing-p (kind name)
+  (ecase kind
+    (:function
+     (let ((macro-fun (info :function :compiler-macro-function name)))
+       (and (closurep macro-fun)
+            (eq (%fun-name macro-fun) '.deprecation-warning.)
+            ;; If you name a function literally :EARLY and it happens to
+            ;; be in :LATE deprecation, then this could be wrong; etc.
+            ;; But come on now ... who would name a function like that?
+            (find-if-in-closure (lambda (x) (member x '(:early :late :final)))
+                                macro-fun))))))
+
+;; This is the moral equivalent of a warning from /usr/bin/ld that
+;; "gets() is dangerous." You're informed by both the compiler and linker.
+(defun loader-deprecation-warn (stuff whence)
+  ;; Stuff is a list: ((<state> name . category) ...)
+  ;; For now we only deal with category = :FUNCTION so we ignore it.
+  (let ((warning-class
+         ;; We're only going to warn once (per toplevel form),
+         ;; so pick the most stern warning applicable.
+         (if (every (lambda (x) (eq (car x) :early)) stuff)
+             'simple-style-warning 'simple-warning)))
+    (warn warning-class
+          :format-control "Reference to deprecated function~P ~S~@[ from ~S~]"
+          :format-arguments
+          (list (length stuff) (mapcar #'second stuff) whence))))
+
 ;;; STATE is one of
 ;;;
 ;;;   :EARLY, for a compile-time style-warning.
@@ -1245,9 +1287,8 @@
              `(progn
                 (declaim (ftype (function * nil) ,name))
                 (setf (fdefinition ',name)
-                      (deprecated-function ,since ',name ',replacements))
-                #!+sb-doc
-                (setf (fdocumentation ',name 'function) ,doc)
+                      (deprecated-function ,since ',name ',replacements
+                                           #!+sb-doc ,doc))
                 ',name)))
        (setf (compiler-macro-function ',name)
              (deprecation-compiler-macro ,state ,since ',name ',replacements)))))
@@ -1599,6 +1640,19 @@ to :INTERPRET, an interpreter will be used.")
        (file-position stream))))
 
 (defun stream-error-position-info (stream &optional position)
+  (when (and (not position) (form-tracking-stream-p stream))
+    (let ((line/col (sb!c::line/col-from-charpos stream)))
+      (return-from stream-error-position-info
+        `((:line ,(car line/col))
+          (:column ,(cdr line/col))
+          ,@(let ((position (file-position-or-nil-for-error stream)))
+              ;; FIXME: 1- is technically broken for multi-byte external
+              ;; encodings, albeit bug-compatible with the broken code in
+              ;; the general case (below) for non-form-tracking-streams.
+              ;; i.e. If you position to this byte, it might not be the
+              ;; first byte of any character.
+              (when position `((:file-position ,(1- position)))))))))
+
   ;; Give up early for interactive streams and non-character stream.
   (when (or (ignore-errors (interactive-stream-p stream))
             (not (subtypep (ignore-errors (stream-element-type stream))
