@@ -2,19 +2,15 @@
 
 (in-package "SB!FASL")
 
-;;; Sometimes we want to skip over any FOPs with side-effects (like
-;;; function calls) while executing other FOPs. *SKIP-UNTIL* will
-;;; either contain the position where the skipping will stop, or
-;;; NIL if we're executing normally.
-(defvar *skip-until* nil)
-
 ;;; Bind STACK-VAR and PTR-VAR to the start of a subsequence of
 ;;; the fop stack of length COUNT, then execute BODY.
 ;;; Within the body, FOP-STACK-REF is used in lieu of SVREF
 ;;; to elide bounds checking.
-(defmacro with-fop-stack ((stack-var ptr-var count) &body body)
-  `(multiple-value-bind (,stack-var ,ptr-var)
-       (truly-the (values simple-vector index) (fop-stack-pop-n ,count))
+(defmacro with-fop-stack (((stack-var &optional stack-expr) ptr-var count)
+                          &body body)
+  `(let* (,@(when stack-expr
+              (list `(,stack-var (the simple-vector ,stack-expr))))
+          (,ptr-var (truly-the index (fop-stack-pop-n ,stack-var ,count))))
      (macrolet ((fop-stack-ref (i)
                   `(locally
                        #-sb-xc-host
@@ -40,15 +36,21 @@
           (values nil arglist))
     (assert (<= (length operands) 2))
     `(progn
-       (defun ,name ,operands
-         ,@(if (null stack-args)
-               forms
-               (with-unique-names (stack ptr)
-                 `((with-fop-stack (,stack ,ptr ,(length stack-args))
-                     (multiple-value-bind ,stack-args
-                         (values ,@(loop for i below (length stack-args)
-                                         collect `(fop-stack-ref (+ ,ptr ,i))))
-                       ,@forms))))))
+       (defun ,name (.fasl-input. ,@operands)
+         (declare (ignorable .fasl-input.))
+         (macrolet ((fasl-input () '(truly-the fasl-input .fasl-input.))
+                    (fasl-input-stream () '(%fasl-input-stream (fasl-input)))
+                    (operand-stack () '(%fasl-input-stack (fasl-input)))
+                    (skip-until () '(%fasl-input-skip-until (fasl-input))))
+           ,@(if (null stack-args)
+                 forms
+                 (with-unique-names (stack ptr)
+                   `((with-fop-stack ((,stack (operand-stack))
+                                      ,ptr ,(length stack-args))
+                       (multiple-value-bind ,stack-args
+                           (values ,@(loop for i below (length stack-args)
+                                           collect `(fop-stack-ref (+ ,ptr ,i))))
+                         ,@forms)))))))
        (!%define-fop ',name ,fop-code ,(length operands) ,(if pushp 1 0)))))
 
 (defun !%define-fop (name base-opcode n-operands pushp)
@@ -57,8 +59,8 @@
     (unless (zerop (mod base-opcode n-slots))
       (error "Opcode for fop ~S must be a multiple of ~D" name n-slots))
     (loop for opcode from base-opcode below (+ base-opcode n-slots)
-          when (functionp (svref *fop-funs* opcode))
-          do (let ((oname (svref *fop-names* opcode)))
+          when (functionp (svref **fop-funs** opcode))
+          do (let ((oname (svref **fop-names** opcode)))
                (when (and oname (not (eq oname name)))
                  (error "fop ~S with opcode ~D conflicts with fop ~S."
                         name opcode oname))))
@@ -72,10 +74,10 @@
     ;; 2-operand fops occupy 16 slots in a reserved range of the function table.
     (dotimes (j n-slots)
       (let ((opcode (+ base-opcode j)))
-        (setf (svref *fop-names* opcode) name
-              (svref *fop-funs* opcode) (symbol-function name)
-              (sbit (car *fop-signatures*) (ash opcode -2)) (signum n-operands)
-              (sbit (cdr *fop-signatures*) opcode) pushp))))
+        (setf (svref **fop-names** opcode) name
+              (svref **fop-funs** opcode) (symbol-function name)
+              (sbit (car **fop-signatures**) (ash opcode -2)) (signum n-operands)
+              (sbit (cdr **fop-signatures**) opcode) pushp))))
   name)
 
 ;;; a helper function for reading string values from FASL files: sort
@@ -128,17 +130,18 @@
 ;;; unused fop codes, so we add this second NOP, which reads 4
 ;;; arbitrary bytes and discards them.
 (!define-fop 137 (fop-nop4 () nil)
-  (let ((arg (read-arg 4)))
+  (let ((arg (read-arg 4 (fasl-input-stream))))
     (declare (ignorable arg))
     #!+sb-show
     (when *show-fop-nop4-p*
       (format *debug-io* "~&/FOP-NOP4 ARG=~W=#X~X~%" arg arg))))
 
 (!define-fop 0 (fop-nop () nil))
-(!define-fop 1 (fop-pop (x) nil) (push-fop-table x))
+(!define-fop 1 (fop-pop (x) nil) (push-fop-table x (fasl-input)))
 (!define-fop 2 (fop-empty-list) nil)
 (!define-fop 3 (fop-truth) t)
-(!define-fop 4 (fop-push ((:operands index))) (ref-fop-table index))
+(!define-fop 4 (fop-push ((:operands index)))
+  (ref-fop-table (fasl-input) index))
 
 ;;; CMU CL had FOP-POP-FOR-EFFECT as fop 65, but it was never used and seemed
 ;;; to have no possible use.
@@ -158,7 +161,7 @@
          ;; otherwise subtract 1 because the layout consumes a slot.
          (n-data-words (- size sb!vm:instance-data-start)))
     (declare (type index size))
-    (with-fop-stack (stack ptr n-data-words)
+    (with-fop-stack ((stack (operand-stack)) ptr n-data-words)
       (let ((ptr (+ ptr n-data-words)))
         (declare (type index ptr))
         (setf (%instance-layout res) layout)
@@ -191,7 +194,7 @@
 ;; or possibly a hand-written equivalent (however unlikely).
 (!define-fop 68 (fop-allocate-instance (name) nil)
   (let ((instance (allocate-instance (find-class (the symbol name)))))
-    (push-fop-table instance)))
+    (push-fop-table instance (fasl-input))))
 
 ;; Fill in object slots as dictated by the second return value from
 ;; MAKE-LOAD-FORM-SAVING-SLOTS.
@@ -199,8 +202,9 @@
 ;; rather than using read-arg because many calls of this might share
 ;; the list, which must be constructed into the fop-table no matter what.
 (!define-fop 69 (fop-initialize-instance (slot-names obj) nil)
-  (let ((n-slots (pop slot-names)))
-    (multiple-value-bind (stack ptr) (fop-stack-pop-n n-slots)
+  (let* ((n-slots (pop slot-names))
+         (stack (operand-stack))
+         (ptr (fop-stack-pop-n stack n-slots)))
       (dotimes (i n-slots)
         (let ((val (svref stack (+ ptr i)))
               (slot-name (pop slot-names)))
@@ -208,7 +212,7 @@
               ;; SLOT-MAKUNBOUND-USING-CLASS might do something nonstandard.
               (slot-makunbound obj slot-name)
               ;; FIXME: the DEFSETF for this isn't defined until warm load
-              (sb!pcl::set-slot-value obj slot-name val)))))))
+              (sb!pcl::set-slot-value obj slot-name val))))))
 
 (!define-fop 64 (fop-end-group () nil)
   (/show0 "THROWing FASL-GROUP-END")
@@ -219,11 +223,11 @@
 ;;; packages directly instead of piggybacking on the host code.
 
 (!define-fop 62 (fop-verify-table-size () nil)
-  (let ((expected-index (read-word-arg)))
-    (unless (= (get-fop-table-index) expected-index)
+  (let ((expected-index (read-word-arg (fasl-input-stream))))
+    (unless (= (svref (%fasl-input-table (fasl-input)) 0) expected-index)
       (bug "fasl table of improper size"))))
 (!define-fop 63 (fop-verify-empty-stack () nil)
-  (unless (fop-stack-empty-p)
+  (unless (fop-stack-empty-p (operand-stack))
     (bug "fasl stack not empty when it should be")))
 
 ;;;; fops for loading symbols
@@ -234,17 +238,18 @@
 
 (declaim (freeze-type undefined-package))
 
-(defun aux-fop-intern (size package)
+(defun aux-fop-intern (size package fasl-input)
   (declare (optimize speed))
-  (let ((buffer (make-string size)))
+  (let ((input-stream (%fasl-input-stream fasl-input))
+        (buffer (make-string size)))
     #+sb-xc-host
-    (read-string-as-bytes *fasl-input-stream* buffer size)
+    (read-string-as-bytes input-stream buffer size)
     #-sb-xc-host
     (progn
       #!+sb-unicode
-      (read-string-as-unsigned-byte-32 *fasl-input-stream* buffer size)
+      (read-string-as-unsigned-byte-32 input-stream buffer size)
       #!-sb-unicode
-      (read-string-as-bytes *fasl-input-stream* buffer size))
+      (read-string-as-bytes input-stream buffer size))
     (if (undefined-package-p package)
         (error 'simple-package-error
                :format-control "Error finding package for symbol ~s:~% ~a"
@@ -255,12 +260,12 @@
                           (intern* buffer
                                    size
                                    package
-                                   :no-copy t))))))
+                                   :no-copy t)) fasl-input))))
 
 (!define-fop 80 (fop-lisp-symbol-save ((:operands namelen)))
-  (aux-fop-intern namelen *cl-package*))
+  (aux-fop-intern namelen *cl-package* (fasl-input)))
 (!define-fop 84 (fop-keyword-symbol-save ((:operands namelen)))
-  (aux-fop-intern namelen *keyword-package*))
+  (aux-fop-intern namelen *keyword-package* (fasl-input)))
 
 ;; But srsly? Most of the space is wasted by UCS4 encoding of ASCII.
 ;; An extra word per symbol for the package is nothing by comparison.
@@ -270,18 +275,19 @@
   ;; FOP-SYMBOL-IN-LAST-PACKAGE-SAVE/FOP-SMALL-SYMBOL-IN-LAST-PACKAGE-SAVE
   ;; cloned fop pair could undo some of this bloat.
 (!define-fop #xF0 (fop-symbol-in-package-save ((:operands pkg-index namelen)))
-  (aux-fop-intern namelen (ref-fop-table pkg-index)))
+  (aux-fop-intern namelen (ref-fop-table (fasl-input) pkg-index) (fasl-input)))
 
 (!define-fop 96 (fop-uninterned-symbol-save ((:operands namelen)))
   (let ((res (make-string namelen)))
     #!-sb-unicode
-    (read-string-as-bytes *fasl-input-stream* res)
+    (read-string-as-bytes (fasl-input-stream) res)
     #!+sb-unicode
-    (read-string-as-unsigned-byte-32 *fasl-input-stream* res)
-    (push-fop-table (make-symbol res))))
+    (read-string-as-unsigned-byte-32 (fasl-input-stream) res)
+    (push-fop-table (make-symbol res) (fasl-input))))
 
 (!define-fop 104 (fop-copy-symbol-save ((:operands table-index)))
-  (push-fop-table (copy-symbol (ref-fop-table table-index))))
+  (push-fop-table (copy-symbol (ref-fop-table (fasl-input) table-index))
+                  (fasl-input)))
 
 (!define-fop 44 (fop-package (pkg-designator))
   (find-undeleted-package-or-lose pkg-designator))
@@ -289,25 +295,26 @@
 (!define-fop 156 (fop-named-package-save ((:operands length)) nil)
   (let ((package-name (make-string length)))
     #+sb-xc-host
-    (read-string-as-bytes *fasl-input-stream* package-name)
+    (read-string-as-bytes (fasl-input-stream) package-name)
     #-sb-xc-host
     (progn
       #!-sb-unicode
-      (read-string-as-bytes *fasl-input-stream* package-name)
+      (read-string-as-bytes (fasl-input-stream) package-name)
       #!+sb-unicode
-      (read-string-as-unsigned-byte-32 *fasl-input-stream* package-name))
+      (read-string-as-unsigned-byte-32 (fasl-input-stream) package-name))
     (push-fop-table
      (handler-case (find-undeleted-package-or-lose package-name)
        (simple-package-error (c)
-         (make-undefined-package :error (princ-to-string c)))))))
+         (make-undefined-package :error (princ-to-string c))))
+     (fasl-input))))
 
 ;;;; fops for loading numbers
 
-;;; Load a signed integer LENGTH bytes long from *FASL-INPUT-STREAM*.
-(defun load-s-integer (length)
+;;; Load a signed integer LENGTH bytes long from FASL-INPUT-STREAM.
+(defun load-s-integer (length fasl-input-stream)
   (declare (fixnum length)
            (optimize speed))
-  (with-fast-read-byte ((unsigned-byte 8) *fasl-input-stream*)
+  (with-fast-read-byte ((unsigned-byte 8) fasl-input-stream)
     (do* ((index length (1- index))
           (byte 0 (fast-read-byte))
           (result 0 (+ result (ash byte bits)))
@@ -319,14 +326,16 @@
       (declare (fixnum index byte bits)))))
 
 (!define-fop 36 (fop-integer ((:operands n-bytes)))
-  (load-s-integer n-bytes))
+  (load-s-integer n-bytes (fasl-input-stream)))
 
 (!define-fop 34 (fop-word-integer)
-  (with-fast-read-byte ((unsigned-byte 8) *fasl-input-stream*)
+  (with-fast-read-byte ((unsigned-byte 8) (fasl-input-stream))
     (fast-read-s-integer #.sb!vm:n-word-bytes)))
 
 (!define-fop 35 (fop-byte-integer)
-  (with-fast-read-byte ((unsigned-byte 8) *fasl-input-stream*)
+  ;; FIXME: WITH-FAST-READ-BYTE for exactly 1 byte is not really faster/better
+  ;; than regular READ-BYTE. The expansion of READ-ARG corroborates this claim.
+  (with-fast-read-byte ((unsigned-byte 8) (fasl-input-stream))
     (fast-read-s-integer 1)))
 
 (!define-fop 70 (fop-ratio (num den)) (%make-ratio num den))
@@ -342,12 +351,12 @@
   (macrolet ((define-complex-fop (opcode name type)
                (let ((reader (symbolicate "FAST-READ-" type)))
                  `(!define-fop ,opcode (,name)
-                      (with-fast-read-byte ((unsigned-byte 8) *fasl-input-stream*)
+                      (with-fast-read-byte ((unsigned-byte 8) (fasl-input-stream))
                         (complex (,reader) (,reader))))))
              (define-float-fop (opcode name type)
                (let ((reader (symbolicate "FAST-READ-" type)))
                  `(!define-fop ,opcode (,name)
-                    (with-fast-read-byte ((unsigned-byte 8) *fasl-input-stream*)
+                    (with-fast-read-byte ((unsigned-byte 8) (fasl-input-stream))
                       (,reader))))))
     (define-complex-fop 72 fop-complex-single-float single-float)
     (define-complex-fop 73 fop-complex-double-float double-float)
@@ -360,28 +369,30 @@
 
 #!+sb-simd-pack
 (!define-fop 88 (fop-simd-pack)
-  (with-fast-read-byte ((unsigned-byte 8) *fasl-input-stream*)
+  (with-fast-read-byte ((unsigned-byte 8) (fasl-input-stream))
     (%make-simd-pack (fast-read-s-integer 8)
                      (fast-read-u-integer 8)
                      (fast-read-u-integer 8))))
 
 ;;;; loading lists
 
-(defun fop-list-from-stack (n)
+(defun fop-list-from-stack (stack n)
   ;; N is 0-255 when called from FOP-LIST,
   ;; but it is as large as ARRAY-RANK-LIMIT in FOP-ARRAY.
   (declare (type (unsigned-byte 16) n)
            (optimize (speed 3)))
-  (with-fop-stack (stack ptr n)
+  (with-fop-stack ((stack) ptr n)
     (do* ((i (+ ptr n) (1- i))
           (res () (cons (fop-stack-ref i) res)))
          ((= i ptr) res)
       (declare (type index i)))))
 
-(!define-fop 33 (fop-list) (fop-list-from-stack (read-byte-arg)))
+(!define-fop 33 (fop-list)
+  (fop-list-from-stack (operand-stack) (read-byte-arg (fasl-input-stream))))
 (!define-fop 16 (fop-list*)
-  (let ((n (read-byte-arg))) ; N is the number of cons cells (0 is ok)
-    (with-fop-stack (stack ptr (1+ n))
+  ;; N is the number of cons cells (0 is ok)
+  (let ((n (read-byte-arg (fasl-input-stream))))
+    (with-fop-stack ((stack (operand-stack)) ptr (1+ n))
       (do* ((i (+ ptr n) (1- i))
             (res (fop-stack-ref (+ ptr n))
                  (cons (fop-stack-ref i) res)))
@@ -414,7 +425,7 @@
 
 (!define-fop 100 (fop-base-string ((:operands length)))
   (let ((res (make-string length :element-type 'base-char)))
-    (read-base-string-as-bytes *fasl-input-stream* res)
+    (read-base-string-as-bytes (fasl-input-stream) res)
     res))
 
 #!+sb-unicode
@@ -428,24 +439,27 @@
   #-sb-xc-host
   (!define-fop 160 (fop-character-string ((:operands length)))
     (let ((res (make-string length)))
-      (read-string-as-unsigned-byte-32 *fasl-input-stream* res)
+      (read-string-as-unsigned-byte-32 (fasl-input-stream) res)
       res)))
 
 (!define-fop 92 (fop-vector ((:operands size)))
-  (let ((res (make-array size)))
+  (let ((res (make-array size))
+        (stack (operand-stack)))
     (declare (fixnum size))
     (unless (zerop size)
-      (multiple-value-bind (stack ptr) (fop-stack-pop-n size)
+      (let ((ptr (fop-stack-pop-n stack size)))
         (replace res stack :start2 ptr)))
     res))
 
 (!define-fop 89 (fop-array (vec))
-  (let* ((rank (read-word-arg))
+  (let* ((rank (read-word-arg (fasl-input-stream)))
          (length (length vec))
          (res (make-array-header sb!vm:simple-array-widetag rank)))
     (declare (simple-array vec)
              (type (unsigned-byte #.(- sb!vm:n-word-bits sb!vm:n-widetag-bits)) rank))
-    (set-array-header res vec length nil 0 (fop-list-from-stack rank) nil t)
+    (set-array-header res vec length nil 0
+                      (fop-list-from-stack (operand-stack) rank)
+                      nil t)
     res))
 
 (defglobal **saetp-bits-per-length**
@@ -463,8 +477,8 @@
                **saetp-bits-per-length**))
 
 (!define-fop 43 (fop-spec-vector)
-  (let* ((length (read-word-arg))
-         (widetag (read-byte-arg))
+  (let* ((length (read-word-arg (fasl-input-stream)))
+         (widetag (read-byte-arg (fasl-input-stream)))
          (bits-per-length (aref **saetp-bits-per-length** widetag))
          (bits (progn (aver (< bits-per-length 255))
                       (* length bits-per-length)))
@@ -473,23 +487,23 @@
          (vector (allocate-vector widetag length words)))
     (declare (type index length bytes words)
              (type word bits))
-    (read-n-bytes *fasl-input-stream* vector 0 bytes)
+    (read-n-bytes (fasl-input-stream) vector 0 bytes)
     vector))
 
 (!define-fop 53 (fop-eval (expr)) ; This seems to be unused
-  (if *skip-until*
+  (if (skip-until)
       expr
       (eval expr)))
 
 (!define-fop 54 (fop-eval-for-effect (expr) nil) ; This seems to be unused
-  (unless *skip-until*
+  (unless (skip-until)
     (eval expr))
   nil)
 
-(defun fop-funcall* ()
- (let ((argc (read-byte-arg)))
-   (with-fop-stack (stack ptr (1+ argc))
-     (unless *skip-until*
+(defun fop-funcall* (input-stream stack skipping)
+ (let ((argc (read-byte-arg input-stream)))
+   (with-fop-stack ((stack) ptr (1+ argc))
+     (unless skipping
        (do ((i (+ ptr argc))
             (args))
            ((= i ptr) (apply (fop-stack-ref i) args))
@@ -497,38 +511,42 @@
          (push (fop-stack-ref i) args)
          (decf i))))))
 
-(!define-fop 55 (fop-funcall) (fop-funcall*))
-(!define-fop 56 (fop-funcall-for-effect () nil) (fop-funcall*))
+;; FIXME: there should be a syntax to share these identical bodies
+(!define-fop 55 (fop-funcall)
+  (fop-funcall* (fasl-input-stream) (operand-stack) (skip-until)))
+(!define-fop 56 (fop-funcall-for-effect () nil)
+  (fop-funcall* (fasl-input-stream) (operand-stack) (skip-until)))
 
 ;;;; fops for fixing up circularities
 
 (!define-fop 200 (fop-rplaca (val) nil)
-  (let ((obj (ref-fop-table (read-word-arg)))
-        (idx (read-word-arg)))
+  (let ((obj (ref-fop-table (fasl-input) (read-word-arg (fasl-input-stream))))
+        (idx (read-word-arg (fasl-input-stream))))
     (setf (car (nthcdr idx obj)) val)))
 
 (!define-fop 201 (fop-rplacd (val) nil)
-  (let ((obj (ref-fop-table (read-word-arg)))
-        (idx (read-word-arg)))
+  (let ((obj (ref-fop-table (fasl-input) (read-word-arg (fasl-input-stream))))
+        (idx (read-word-arg (fasl-input-stream))))
     (setf (cdr (nthcdr idx obj)) val)))
 
 (!define-fop 202 (fop-svset (val) nil)
-  (let* ((obi (read-word-arg))
-         (obj (ref-fop-table obi))
-         (idx (read-word-arg)))
+  (let* ((obi (read-word-arg (fasl-input-stream)))
+         (obj (ref-fop-table (fasl-input) obi))
+         (idx (read-word-arg (fasl-input-stream))))
     (if (%instancep obj) ; suspicious. should have been FOP-STRUCTSET
         (setf (%instance-ref obj idx) val)
         (setf (svref obj idx) val))))
 
 (!define-fop 204 (fop-structset (val) nil)
-  (setf (%instance-ref (ref-fop-table (read-word-arg))
-                       (read-word-arg))
+  (setf (%instance-ref (ref-fop-table (fasl-input)
+                                      (read-word-arg (fasl-input-stream)))
+                       (read-word-arg (fasl-input-stream)))
         val))
 
 ;;; In the original CMUCL code, this actually explicitly declared PUSHP
 ;;; to be T, even though that's what it defaults to in DEFINE-FOP.
 (!define-fop 203 (fop-nthcdr (obj))
-  (nthcdr (read-word-arg) obj))
+  (nthcdr (read-word-arg (fasl-input-stream)) obj))
 
 ;;;; fops for loading functions
 
@@ -540,10 +558,14 @@
 ;;; fasl file header.)
 
 (!define-fop #xE0 (fop-code ((:operands n-boxed-words n-unboxed-bytes)))
-  (load-code n-boxed-words n-unboxed-bytes))
+  (with-fop-stack ((stack (operand-stack)) ptr (1+ n-boxed-words))
+    (load-code n-boxed-words n-unboxed-bytes stack ptr (fasl-input-stream))))
 
 ;; this gets you an #<fdefn> object, not the result of (FDEFINITION x)
 (!define-fop 60 (fop-fdefn (name))
+  (awhen (deprecated-thing-p :function name) ; returns the stage of deprecation
+    (pushnew (list* it name :function)
+             (%fasl-input-deprecated-stuff (fasl-input)) :test 'equal))
   (find-or-create-fdefn name))
 
 (!define-fop 65 (fop-known-fun (name))
@@ -554,31 +576,11 @@
   (sb!vm:sanctify-for-execution component)
   component)
 
-(!define-fop 74 (fop-fset (name fn) nil)
-  ;; Ordinary, not-for-cold-load code shouldn't need to mess with this
-  ;; at all, since it's only used as part of the conspiracy between
-  ;; the cross-compiler and GENESIS to statically link FDEFINITIONs
-  ;; for cold init.
-  (warn "~@<FOP-FSET seen in ordinary load (not cold load) -- quite strange! ~
-If you didn't do something strange to cause this, please report it as a ~
-bug.~:@>")
-  ;; Unlike CMU CL, we don't treat this as a no-op in ordinary code.
-  ;; If the user (or, more likely, developer) is trying to reload
-  ;; compiled-for-cold-load code into a warm SBCL, we'll do a warm
-  ;; assignment. (This is partly for abstract tidiness, since the warm
-  ;; assignment is the closest analogy to what happens at cold load,
-  ;; and partly because otherwise our compiled-for-cold-load code will
-  ;; fail, since in SBCL things like compiled-for-cold-load %DEFUN
-  ;; depend more strongly than in CMU CL on FOP-FSET actually doing
-  ;; something.)
-  (setf (fdefinition name) fn))
-
 (!define-fop 174 (fop-note-debug-source (debug-source) nil)
   (warn "~@<FOP-NOTE-DEBUG-SOURCE seen in ordinary load (not cold load) -- ~
 very strange!  If you didn't do something to cause this, please report it as ~
 a bug.~@:>")
-  ;; as with COLD-FSET above, we are going to be lenient with coming
-  ;; across this fop in a warm SBCL.
+  ;; we are going to be lenient with coming across this fop in a warm SBCL.
   (setf (sb!c::debug-source-compiled debug-source) (get-universal-time)
         (sb!c::debug-source-created debug-source)
         (file-write-date (sb!c::debug-source-namestring debug-source))))
@@ -592,7 +594,7 @@ a bug.~@:>")
   #+sb-xc-host ; since xc host doesn't know how to compile %PRIMITIVE
   (error "FOP-FUN-ENTRY can't be defined without %PRIMITIVE.")
   #-sb-xc-host
-  (let ((offset (read-word-arg)))
+  (let ((offset (read-word-arg (fasl-input-stream))))
     (declare (type index offset))
     (unless (zerop (logand offset sb!vm:lowtag-mask))
       (bug "unaligned function object, offset = #X~X" offset))
@@ -628,43 +630,44 @@ a bug.~@:>")
 
 (!define-fop 146 (fop-symbol-tls-fixup (code-object kind symbol))
   (sb!vm:fixup-code-object code-object
-                           (read-word-arg)
+                           (read-word-arg (fasl-input-stream))
                            (ensure-symbol-tls-index symbol)
                            kind)
   code-object)
 
 (!define-fop 147 (fop-foreign-fixup (code-object kind))
-  (let* ((len (read-byte-arg))
+  (let* ((len (read-byte-arg (fasl-input-stream)))
          (sym (make-string len :element-type 'base-char)))
-    (read-n-bytes *fasl-input-stream* sym 0 len)
+    (read-n-bytes (fasl-input-stream) sym 0 len)
     (sb!vm:fixup-code-object code-object
-                             (read-word-arg)
+                             (read-word-arg (fasl-input-stream))
                              (foreign-symbol-address sym)
                              kind)
     code-object))
 
 (!define-fop 148 (fop-assembler-fixup (code-object kind routine))
-    (multiple-value-bind (value found) (gethash routine *assembler-routines*)
-      (unless found
-        (error "undefined assembler routine: ~S" routine))
-      (sb!vm:fixup-code-object code-object (read-word-arg) value kind))
-    code-object)
+  (multiple-value-bind (value found) (gethash routine *assembler-routines*)
+    (unless found
+      (error "undefined assembler routine: ~S" routine))
+    (sb!vm:fixup-code-object code-object (read-word-arg (fasl-input-stream))
+                             value kind))
+  code-object)
 
 (!define-fop 149 (fop-code-object-fixup (code-object kind))
     ;; Note: We don't have to worry about GC moving the code-object after
     ;; the GET-LISP-OBJ-ADDRESS and before that value is deposited, because
     ;; we can only use code-object fixups when code-objects don't move.
-    (sb!vm:fixup-code-object code-object (read-word-arg)
-                             (get-lisp-obj-address code-object) kind)
-    code-object)
+  (sb!vm:fixup-code-object code-object (read-word-arg (fasl-input-stream))
+                           (get-lisp-obj-address code-object) kind)
+  code-object)
 
 #!+linkage-table
 (!define-fop 150 (fop-foreign-dataref-fixup (code-object kind))
-  (let* ((len (read-byte-arg))
+  (let* ((len (read-byte-arg (fasl-input-stream)))
          (sym (make-string len :element-type 'base-char)))
-    (read-n-bytes *fasl-input-stream* sym 0 len)
+    (read-n-bytes (fasl-input-stream) sym 0 len)
     (sb!vm:fixup-code-object code-object
-                             (read-word-arg)
+                             (read-word-arg (fasl-input-stream))
                              (foreign-symbol-address sym t)
                              kind)
     code-object))
@@ -677,34 +680,34 @@ a bug.~@:>")
 ;;; be done to ensure that the fop table gets populated correctly
 ;;; regardless of the execution path.
 (!define-fop 151 (fop-skip (position) nil)
-  (unless *skip-until*
-    (setf *skip-until* position))
+  (unless (skip-until)
+    (setf (skip-until) position))
   (values))
 
 ;;; As before, but only start skipping if the top of the FOP stack is NIL.
 (!define-fop 152 (fop-skip-if-false (position condition) nil)
-  (unless (or condition *skip-until*)
-    (setf *skip-until* position))
+  (unless (or condition (skip-until))
+    (setf (skip-until) position))
   (values))
 
 ;;; If skipping, pop the top of the stack and discard it. Needed for
 ;;; ensuring that the stack stays balanced when skipping.
 (!define-fop 153 (fop-drop-if-skipping () nil)
-  (when *skip-until*
-    (fop-stack-pop-n 1))
+  (when (skip-until)
+    (fop-stack-pop-n (operand-stack) 1))
   (values))
 
 ;;; If skipping, push a dummy value on the stack. Needed for
 ;;; ensuring that the stack stays balanced when skipping.
 (!define-fop 154 (fop-push-nil-if-skipping () nil)
-  (when *skip-until*
-    (push-fop-stack nil))
+  (when (skip-until)
+    (push-fop-stack nil (fasl-input)))
   (values))
 
-;;; Stop skipping if the top of the stack matches *SKIP-UNTIL*
+;;; Stop skipping if the top of the stack matches SKIP-UNTIL
 (!define-fop 155 (fop-maybe-stop-skipping (label) nil)
-  (when (eql *skip-until* label)
-    (setf *skip-until* nil))
+  (when (eql (skip-until) label)
+    (setf (skip-until) nil))
   (values))
 
 ;;; Primordial layouts.
