@@ -93,12 +93,11 @@
          (cnt 1 (1+ cnt)))
         ((>= cnt n) res))))
 
-;;; Read an N-byte unsigned integer from the *FASL-INPUT-STREAM*.
-(defmacro read-arg (n)
-  (declare (optimize (speed 0)))
+;;; Read an N-byte unsigned integer from FASL-INPUT-STREAM.
+(defmacro read-arg (n fasl-input-stream)
   (if (= n 1)
-      `(the (unsigned-byte 8) (read-byte *fasl-input-stream*))
-      `(with-fast-read-byte ((unsigned-byte 8) *fasl-input-stream*)
+      `(the (unsigned-byte 8) (read-byte ,fasl-input-stream))
+      `(with-fast-read-byte ((unsigned-byte 8) ,fasl-input-stream)
          (fast-read-u-integer ,n))))
 
 ;; FIXME: on x86-64, these functions exceed 600, 900, and 1200 bytes of code
@@ -108,21 +107,21 @@
 ;; But this would only be feasible on machines that are little-endian
 ;; and that allow unaligned reads. (like x86)
 (declaim (inline read-byte-arg read-halfword-arg read-word-arg))
-(defun read-byte-arg ()
+(defun read-byte-arg (stream)
   (declare (optimize (speed 0)))
-  (read-arg 1))
+  (read-arg 1 stream))
 
-(defun read-halfword-arg ()
+(defun read-halfword-arg (stream)
   (declare (optimize (speed 0)))
-  (read-arg #.(/ sb!vm:n-word-bytes 2)))
+  (read-arg #.(/ sb!vm:n-word-bytes 2) stream)) ; READ-ARG doesn't eval N
 
-(defun read-word-arg ()
+(defun read-word-arg (stream)
   (declare (optimize (speed 0)))
-  (read-arg #.sb!vm:n-word-bytes))
+  (read-arg #.sb!vm:n-word-bytes stream))
 
-(defun read-unsigned-byte-32-arg ()
+(defun read-unsigned-byte-32-arg (stream)
   (declare (optimize (speed 0)))
-  (read-arg 4))
+  (read-arg 4 stream))
 
 
 ;;;; the fop table
@@ -133,28 +132,18 @@
 ;;; FOPs use the table to save stuff, other FOPs refer to the table by
 ;;; direct indexes via REF-FOP-TABLE.
 
-(defvar *fop-table*)
-(declaim (simple-vector *fop-table*))
-
 (declaim (inline ref-fop-table))
-(defun ref-fop-table (index)
-  (declare (type index index))
-  (svref *fop-table* (the index (+ index 1))))
+(defun ref-fop-table (fasl-input index)
+  (svref (%fasl-input-table fasl-input) (1+ (the index index))))
 
-(defun get-fop-table-index ()
-  (svref *fop-table* 0))
-
-(defun reset-fop-table ()
-  (setf (svref *fop-table* 0) 0))
-
-(defun push-fop-table (thing) ; and return THING
-  (let* ((table *fop-table*)
+(defun push-fop-table (thing fasl-input) ; and return THING
+  (let* ((table (%fasl-input-table fasl-input))
          (index (+ (the index (aref table 0)) 1)))
     (declare (fixnum index)
              (simple-vector table))
     (when (eql index (length table))
       (setf table (grow-fop-vector table index)
-            *fop-table* table))
+            (%fasl-input-table fasl-input) table))
     (setf (aref table 0) index
           (aref table index) thing)))
 
@@ -187,35 +176,29 @@
 
 ;;;; the fop stack
 
-;;; Much like the table, this is bound to a simple vector whose first
-;;; element is the current index.
-(defvar *fop-stack*)
-(declaim (simple-vector *fop-stack*))
-
 (declaim (inline fop-stack-empty-p))
-(defun fop-stack-empty-p ()
-  (eql 0 (svref *fop-stack* 0)))
+(defun fop-stack-empty-p (stack)
+  (eql 0 (svref stack 0)))
 
 ;; Ensure that N arguments can be popped from the FOP stack.
 ;; Return the stack and the pointer to the first argument.
 ;; Update the new top-of-stack to reflect that all N have been popped.
-(defun fop-stack-pop-n (n)
+(defun fop-stack-pop-n (stack n)
   (declare (type index n))
-  (let* ((stack *fop-stack*)
-         (top (the index (svref stack 0)))
+  (let* ((top (the index (svref stack 0)))
          (new-top (- top n)))
     (if (minusp new-top) ; 0 is ok at this point
         (error "FOP stack underflow")
         (progn (setf (svref stack 0) new-top)
-               (values stack (1+ new-top))))))
+               (1+ new-top)))))
 
-(defun push-fop-stack (value)
-  (let* ((stack *fop-stack*)
+(defun push-fop-stack (value fasl-input)
+  (let* ((stack (%fasl-input-stack fasl-input))
          (next (1+ (the index (svref stack 0)))))
     (declare (type index next))
     (when (eql (length stack) next)
       (setf stack (grow-fop-vector stack next)
-            *fop-stack* stack))
+            (%fasl-input-stack fasl-input) stack))
     (setf (svref stack 0) next
           (svref stack next) value)))
 
@@ -372,7 +355,7 @@
                    :expected (char-code (schar fhsss count))))))
       ;; Read and validate version-specific compatibility stuff.
       (flet ((string-from-stream ()
-               (let* ((length (read-unsigned-byte-32-arg))
+               (let* ((length (read-unsigned-byte-32-arg stream))
                       (result (make-string length)))
                  (read-string-as-bytes stream result)
                  result)))
@@ -384,7 +367,7 @@
                    :stream stream
                    :implementation implementation
                    :expected expected-implementation)))
-        (let* ((fasl-version (read-word-arg))
+        (let* ((fasl-version (read-word-arg stream))
                (sbcl-version (if (<= fasl-version 76)
                                  "1.0.11.18"
                                  (string-from-stream)))
@@ -419,7 +402,7 @@
 ;;; NIL if EOF was encountered while trying to read from the stream.
 ;;; Dispatch to the right function for each fop.
 (defconstant +2-operand-fops+ #xE0) ; start of the range
-(defun load-fasl-group (stream print)
+(defun load-fasl-group (fasl-input print)
   ;;
   ;; PRINT causes most tlf-equivalent forms to print their primary value.
   ;; This differs from loading of Lisp source, which prints all values of
@@ -430,12 +413,11 @@
   ;;   contents of the source file, but some information is generally printed."
   ;;
   (declare (ignorable print))
-  (unless (check-fasl-header stream)
-    (return-from load-fasl-group))
-  (catch 'fasl-group-end
-    (reset-fop-table)
-    (let ((*skip-until* nil))
-      (declare (special *skip-until*))
+  (let ((stream (%fasl-input-stream fasl-input)))
+    (unless (check-fasl-header stream)
+      (return-from load-fasl-group))
+    (catch 'fasl-group-end
+      (setf (svref (%fasl-input-table fasl-input) 0) 0)
       (loop
        (let ((byte (the (unsigned-byte 8) (read-byte stream)))
              (trace (or #!+sb-show *show-fops-p*)))
@@ -443,17 +425,19 @@
          (when trace
            (format *trace-output* "~&~6x : [~D,~D] ~2,'0x(~A)"
                    (1- (file-position stream))
-                   (svref *fop-stack* 0) ; stack pointer
-                   (svref *fop-table* 0) ; table pointer
-                   byte (aref *fop-names* byte)))
+                   (svref (%fasl-input-stack fasl-input) 0) ; stack pointer
+                   (svref (%fasl-input-table fasl-input) 0) ; table pointer
+                   byte (aref **fop-names** byte)))
          ;; Actually execute the fop.
          (let ((result
-                (let ((function (svref *fop-funs* byte)))
+                (let ((function (svref **fop-funs** byte)))
                   (cond ((not (functionp function))
                          (error "corrupt fasl file: FOP code #x~x" byte))
-                        ((zerop (sbit (car *fop-signatures*) (ash byte -2)))
-                         (funcall function)) ; takes no arguments
+                        ((zerop (sbit (car **fop-signatures**) (ash byte -2)))
+                         ;; takes no arguments from the fasl file
+                         (funcall function fasl-input))
                         (t
+                         ;; one or two arguments come from the fasl file
                          (let (arg1 arg2) ; See !%DEFINE-FOP for encoding
                            (with-fast-read-byte ((unsigned-byte 8) stream)
                              (setq arg1 (fast-read-var-u-integer
@@ -464,40 +448,45 @@
                            (when trace
                              (format *trace-output* "{~D~@[,~D~]}" arg1 arg2))
                            (if arg2
-                               (funcall function arg1 arg2)
-                               (funcall function arg1))))))))
-           (when (plusp (sbit (cdr *fop-signatures*) byte))
-             (push-fop-stack result))
-           (when trace
-             (let* ((stack *fop-stack*)
-                    (ptr (svref stack 0)))
+                               (funcall function fasl-input arg1 arg2)
+                               (funcall function fasl-input arg1))))))))
+           (when (plusp (sbit (cdr **fop-signatures**) byte))
+             (push-fop-stack result fasl-input))
+           (let* ((stack (%fasl-input-stack fasl-input))
+                  (ptr (svref stack 0)))
+             (when trace
                (format *trace-output* " -- ~[<empty>,~D~:;[~:*~D,~D] ~S~]"
-                       ptr (svref *fop-table* 0)
+                       ptr (svref (%fasl-input-table fasl-input) 0)
                        (unless (eql ptr 0) (aref stack ptr)))
-               (terpri *trace-output*)))
-           #-sb-xc-host
-           (macrolet ((terminator-opcode ()
-                        (or (get 'fop-funcall-for-effect 'opcode)
-                            (error "Missing FOP definition?"))))
-             (when (and print
-                        (eq byte (terminator-opcode))
-                        (fop-stack-empty-p)) ; (presumed) end of TLF
-               (load-fresh-line)
-               (prin1 result)))))))))
+               (terpri *trace-output*))
+             #-sb-xc-host
+             (macrolet ((terminator-opcode ()
+                          (or (get 'fop-funcall-for-effect 'opcode)
+                              (error "Missing FOP definition?"))))
+               (when (and (eq byte (terminator-opcode))
+                          (fop-stack-empty-p stack)) ; (presumed) end of TLF
+                 (awhen (%fasl-input-deprecated-stuff fasl-input)
+                   ;; Delaying this message rather than printing it
+                   ;; in fop-fdefn makes it more informative (usually).
+                   (setf (%fasl-input-deprecated-stuff fasl-input) nil)
+                   (loader-deprecation-warn
+                    it
+                    (and (eq (svref stack 1) 'sb!impl::%defun) (svref stack 2))))
+                 (when print
+                   (load-fresh-line)
+                   (prin1 result)))))))))))
 
 (defun load-as-fasl (stream verbose print)
   (when (zerop (file-length stream))
     (error "attempt to load an empty FASL file:~%  ~S" (namestring stream)))
   (maybe-announce-load stream verbose)
-  (let* ((*fasl-input-stream* stream)
-         (*fop-table* (make-fop-vector 1000))
-         (*fop-stack* (make-fop-vector 100)))
+  (let ((fasl-input (make-fasl-input stream)))
     (unwind-protect
-         (loop while (load-fasl-group stream print))
+         (loop while (load-fasl-group fasl-input print))
       ;; Nuke the table and stack to avoid keeping garbage on
       ;; conservatively collected platforms.
-      (nuke-fop-vector *fop-table*)
-      (nuke-fop-vector *fop-stack*)))
+      (nuke-fop-vector (%fasl-input-table fasl-input))
+      (nuke-fop-vector (%fasl-input-stack fasl-input))))
   t)
 
 (declaim (notinline read-byte)) ; Why is it even *declaimed* inline above?
