@@ -162,8 +162,9 @@
           (when (and (or should-be-fasl-p (not (eql 0 (file-length stream))))
                      (fasl-header-p stream :errorp should-be-fasl-p))
             (return-from load (load-stream stream t)))))
-      ;; Case 3: Open using the gived external format, process as source.
-      (with-open-file (stream pathname :external-format external-format)
+      ;; Case 3: Open using the given external format, process as source.
+      (with-open-file (stream pathname :external-format external-format
+                              :class 'form-tracking-stream)
         (load-stream stream nil)))))
 
 ;; This implements the defaulting SBCL seems to have inherited from
@@ -205,72 +206,18 @@
           (defaulted-source-truename defaulted-source-pathname))))
 
 ;;; Load a code object. BOX-NUM objects are popped off the stack for
-;;; the boxed storage section, then SIZE bytes of code are read in.
-#!-x86
-(defun load-code (box-num code-length)
+;;; the boxed storage section, then CODE-LENGTH bytes of code are read in.
+(defun load-code (box-num code-length stack ptr input-stream)
   (declare (fixnum box-num code-length))
-  (with-fop-stack t
-    (let ((code (sb!c:allocate-code-object box-num code-length))
-          (index (+ sb!vm:code-constants-offset box-num)))
-      (declare (type index index))
-      (setf (%code-debug-info code) (pop-stack))
-      (dotimes (i box-num)
-        (declare (fixnum i))
-        (setf (code-header-ref code (decf index)) (pop-stack)))
-      (without-gcing
-        (read-n-bytes *fasl-input-stream*
-                      (code-instructions code)
-                      0
-                      code-length))
-      code)))
-
-;;; Moving native code during a GC or purify is not so trivial on the
-;;; x86 port.
-;;;
-;;; Our strategy for allowing the loading of x86 native code into the
-;;; dynamic heap requires that the addresses of fixups be saved for
-;;; all these code objects. After a purify these fixups can be
-;;; dropped. In CMU CL, this policy was enabled with
-;;; *ENABLE-DYNAMIC-SPACE-CODE*; in SBCL it's always used.
-#!+x86
-(defun load-code (box-num code-length)
-  (declare (fixnum box-num code-length))
-  (with-fop-stack t
-    (let ((stuff (list (pop-stack))))
-      (dotimes (i box-num)
-        (declare (fixnum i))
-        (push (pop-stack) stuff))
-      (let* ((dbi (car (last stuff))))  ; debug-info
-
-        (setq stuff (nreverse stuff))
-
-        ;; FIXME: *LOAD-CODE-VERBOSE* should probably be #!+SB-SHOW.
-        (when *load-code-verbose*
-          (format t "stuff: ~S~%" stuff)
-          (format t
-                  "   : ~S ~S ~S~%"
-                  (sb!c::compiled-debug-info-p dbi)
-                  (sb!c::debug-info-p dbi)
-                  (sb!c::compiled-debug-info-name dbi))
-          (format t "   loading to the dynamic space~%"))
-
-        (let ((code (sb!c:allocate-code-object box-num code-length))
-              (index (+ sb!vm:code-constants-offset box-num)))
-          (declare (type index index))
-          (when *load-code-verbose*
-            (format t
-                    "  obj addr=~X~%"
-                    (get-lisp-obj-address code)))
-          (setf (%code-debug-info code) (pop stuff))
-          (dotimes (i box-num)
-            (declare (fixnum i))
-            (setf (code-header-ref code (decf index)) (pop stuff)))
-          (without-gcing
-            (read-n-bytes *fasl-input-stream*
-                          (code-instructions code)
-                          0
-                          code-length))
-          code)))))
+  (declare (simple-vector stack) (type index ptr))
+  (let ((code (sb!c:allocate-code-object box-num code-length)))
+    (setf (%code-debug-info code) (svref stack (+ ptr box-num)))
+    (loop for i of-type index from sb!vm:code-constants-offset
+          for j of-type index from ptr below (+ ptr box-num)
+          do (setf (code-header-ref code i) (svref stack j)))
+    (without-gcing
+      (read-n-bytes input-stream (code-instructions code) 0 code-length))
+    code))
 
 ;;;; linkage fixups
 
