@@ -664,13 +664,15 @@
                    (:test callable) (:test-not callable) (:start index)
                    (:from-end t) (:end sequence-end) (:key callable))
   index
-  (foldable flushable call))
+  (foldable flushable call)
+  :derive-type #'count-derive-type)
 
 (defknown (count-if count-if-not)
   (callable sequence &rest t &key
             (:from-end t) (:start index) (:end sequence-end) (:key callable))
   index
-  (foldable flushable call))
+  (foldable flushable call)
+  :derive-type #'count-derive-type)
 
 (defknown (mismatch search)
   (sequence sequence &rest t &key (:from-end t) (:test callable)
@@ -1359,6 +1361,7 @@
 
 (defknown open
   (pathname-designator &key
+                       (:class symbol)
                        (:direction (member :input :output :io :probe))
                        (:element-type type-specifier)
                        (:if-exists (member :error :new-version :rename
